'' =================================================================================================
''
''   File....... jm_simple_rtc_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 24 DEC 2014
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                       
  _xinfreq = 5_000_000                                           ' use 5MHz crystal
                                                                  
  CLK_FREQ = _xinfreq * (_clkmode >> 6)                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                                                                  
                                                                  
con { io pins }                                                   
                                                                  
  RX1 = 31                                                       ' programming / terminal
  TX1 = 30                                                        
                                                                  
  SDA = 29                                                       ' eeprom / i2c
  SCL = 28


obj

  term : "jm_fullduplexserial"
 

var


pub main

  setup

  repeat
    term.tx(term#HOME)
    term.rjdec(hrs, 2, "0")
    term.tx(":")
    term.rjdec(mns, 2, "0")
    term.tx(":")
    term.rjdec(scs, 2, "0")
    term.tx(".")    
    term.rjdec(millis, 3, "0")

    pause(10)                                                    ' prevent terminal buffer overrun


pub setup

'' Setup IO and objects for application

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal
  pause(5)

  start_rtc                                                      ' start rtc cog           


con

  { --------------------- }
  {  S I M P L E   R T C  }
  { --------------------- }


con

  #0, RTC_MS, RTC_SC, RTC_MN, RTC_HR
  

var

  long  rtccog
  long  rtcstack[16]

  long  rtchold

  long  millis
  long  scs
  long  mns
  long  hrs


pri start_rtc

  stop_rtc
  rtccog := cognew(run_rtc, @rtcstack) + 1

  return rtccog


pri stop_rtc

  if (rtccog)
    cogstop(rtccog-1)
    rtccog := 0

  longfill(@millis, 0, 4)
    

pri run_rtc | mstix, t                                           ' lauch with cognew

  mstix := clkfreq / 1000                                        ' tix in 1ms

  t := cnt                                                       ' sync timebase
  repeat
    repeat while (rtchold)                                       ' if rtc on hold
      t := cnt                                                   '  re-sync

    waitcnt(t += mstix)                                          ' wait 1ms

    if (++millis == 1000)                                        ' update milliseconds
      millis := 0
      if (++scs == 60)                                           ' update seconds if needed
        scs := 0
        if (++mns == 60)                                         ' update minutes if needed
          mns := 0
          if (++hrs == 24)                                       ' update hours if needed
            hrs := 0


con

  { ------------- }
  {  B A S I C S  }
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  